package org.tbasket.handler

import io.netty.handler.codec.http.QueryStringDecoder
import org.tbasket.error.*
import zio.http.Body
import zio.http.api.openapi.OpenAPI.Parameter.QueryStyle.Form
import zio.json.*
import zio.json.ast.{Json, JsonCursor}
import zio.{Task, ZIO}

import java.net.URLDecoder
import java.nio.charset.StandardCharsets
import scala.jdk.CollectionConverters.*
import scala.language.reflectiveCalls

object HandlerUtils {
  def parseAttribute[V, T <: Json {def value: V}](json: Json, name: String, cursor: JsonCursor[Json, T]): Task[V] =
    ZIO.fromEither(json.get[T](cursor).map(_.value))
      .mapError(InvalidRequest(s"Missing or invalid field $name.", _))
  
  def parseAttributeOpt[V, T <: Json {def value: V}](json: Json, name: String, cursor: JsonCursor[Json, T]) =
    ZIO.fromEither(json.get[T](cursor).map(_.value)).option
  
  
  def errorBody(errorType: String, msg: String) = Body.fromString(s"""{"error": "$errorType","msg": "$msg"}""")
  
  def parseRequestForm(body: Body) =
    (for
      decoded <- body.asString.map(URLDecoder.decode(_, StandardCharsets.UTF_8.name()))
      params = decoded.split('&').collect { case s"$k=$v" => (k, v)}.toMap
    yield params)
      .mapError(s => InvalidRequest("Invalid request body", s.getMessage))
  
  def search(map: Map[String, String])(name: String) =
    ZIO.attempt(map.get(name)).someOrFail(InvalidRequest(s"Missing or invalid field $name."))
  
}
